/***************************************************************/
/* File Id:                     cal.c.                         */
/* Author:                      Stan Milam.                    */
/* Date Written:                25-Mar-1994.                   */
/*                                                             */
/* This program uses the extended date functions to implement a*/
/* calendar.  If no argument is entered on the command line a  */
/* calendar for the current month is displayed.  Otherwise a   */
/* calendar for the month and year entered on the command line */
/* is displayed.                                               */
/*                                                             */
/***************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <toolbox.h>

int main ( int argc, char **argv ) {

    date_t wrk;
    struct dt dt;
    char   buffer[25];
    short  x_sub, y_sub, last_day, first_day, status;

    /***********************************************************/
    /* Get the current date, the first day of the month, the   */
    /* last day of the month and a date structure for the 1st  */
    /* day of the month.                                       */
    /***********************************************************/

    wrk = date(NULL);
    dt = *localdate(&wrk);

    if ( argc > 1 ) {
        argv++;
        status = sscanf(*argv,"%d%*c%d", &dt.dt_month, &dt.dt_year);
        if ( status > 0 ) {
            dt.dt_month--;
            if ((wrk = mkdate( &dt )) == -1L) return EXIT_FAILURE;
        }
        else return EXIT_FAILURE;
    }

    /***********************************************************/
    /* Compute the first day of the month so that we that we   */
    /* learn which day of the week the first day is on.        */
    /***********************************************************/

    wrk = first_day_of_month( wrk );
    dt = *localdate( &wrk );
    first_day = dt.dt_wday;

    /***********************************************************/
    /* Get the last day of the month so we know where to stop. */
    /***********************************************************/

    wrk = last_day_of_month( wrk );
    dt  = *localdate( &wrk );
    last_day = dt.dt_mday;

    /***********************************************************/
    /* Format the month and year we are going to print and     */
    /* display the heading to the calendar.                    */
    /***********************************************************/

    strfdate(buffer, sizeof(buffer), "%B, %Y\n", &dt);
    printf(buffer);
    printf("Sun Mon Tue Wes Thu Fri Sat \n");
    printf("=========================== \n");

    /***********************************************************/
    /* Now determine where to start printing the first number. */
    /***********************************************************/

    for (x_sub = SUNDAY; x_sub < first_day; x_sub++)
        printf("    ");

    /***********************************************************/
    /* Now enter a loop printing the numbers corresponding to  */
    /* the day of the week.  Must handle wrap around.          */
    /***********************************************************/

    for (x_sub = 1; x_sub <= last_day; x_sub++) {
        printf("%3d ", x_sub);
        if ( ++first_day % 7 == 0 ) printf("\n");
    }
    return 0;
}
