/***************************************************************/
/* File Id:                     dates.h.                       */
/* Author:                      Stan Milam.                    */
/* Date Written:                22 Sep. 93.                    */
/*                                                             */
/* This header file establishes the dt structure type and      */
/* declares several prototypes for custom date routines that   */
/* are similar to the C library time.h routines.               */
/*                                                             */
/***************************************************************/

#ifndef DATE_T
#  define DATE_T 1
#  ifdef __cplusplus
   extern "C" {
#  endif

/***************************************************************/
/* Define values for the days of the week.                     */
/***************************************************************/

#define SUNDAY           0
#define MONDAY           1
#define TUESDAY          2
#define WEDNESDAY        3
#define THURSDAY         4
#define FRIDAY           5
#define SATURDAY         6

/***************************************************************/
/* Define the values used by to_date() & to_char() for date    */
/* strings.                                                    */
/***************************************************************/

#define JULDATE          1             /* CCYYDDD */
#define GREGDATE         2             /* MM/DD/CCYY */
#define SPELLDATE        3             /* DD-MMM-CCYY */
#define MILDATE          4             /* DD/MM/CCYY */
#define SYSDATE          5             /* CCYYMMDD */
#define DDMMCCYY         6
#define MMDDCCYY         7

typedef long date_t;

struct dt {
    int dt_leap_year;                  /* Indicates a leap year */
    int dt_year;                       /* The actual year */
    int dt_month;                      /* The month, 0 - 11 */
    int dt_mday;                       /* The day of month, 1-31 */
    int dt_yday;                       /* The day of the year, 0-365 */
    int dt_wday;                       /* The day of the week, 0-6, 0=Sun*/
};

date_t date( date_t * );
date_t mkdate ( struct dt * );
date_t time_to_date( long tv );
struct dt *localdate( date_t * );
date_t last_day_of_month ( date_t value );
date_t first_day_of_month( date_t value );
date_t to_date( char *string, int type );
date_t next_day_of_week(date_t date_val, int day);
date_t previous_day_of_week(date_t value, int day );
char * date_to_string( char *buffer, date_t day, int type);
char * to_char( char *buffer, date_t day, int type );
unsigned strfdate( char *buf, unsigned size, char *format, struct dt *dt );
date_t compute_date(date_t start, int years, int months, int weeks, int days);

#   ifdef __cplusplus
}
#   endif
#endif
