/***************************************************************/
/* Program Id:                  datetest.c.                    */
/* Author:                      Stan Milam.                    */
/* Date Written:                1-Dec-1993.                    */
/*                                                             */
/* This program should test all of the functions in the dates  */
/* library for functionality.  However, some bugs could still  */
/* lurk around in there somewhere.                             */
/*                                                             */
/***************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "dates.h"

date_t previous_day( date_t, int );

int main( void ) {

    int    i;
    struct dt dt;
    time_t time_val;
    date_t dv1, dv2;
    char   buffer[80];

    /***********************************************************/
    /* First check for errors.  All these calls should fail.   */
    /***********************************************************/

    dv1 = to_date("13/1/93", GREGDATE);
    printf("13/1/93 %s to_date()\n", dv1 == -1L ? "failed" : "passed");

    dv1 = to_date("1/32/93", GREGDATE);
    printf("1/32/93 %s to_date()\n", dv1 == -1L ? "failed" : "passed");

    dv1 = to_date("1/1/10000", GREGDATE);
    printf("1/1/10000 %s to_date()\n", dv1 == -1L ? "failed" : "passed");

    dv1 = to_date("93366", JULDATE);
    printf("93366 %s to_date()\n", dv1 == -1L ? "failed" : "passed" );

    /***********************************************************/
    /* Try out the date() function with date_to_string().      */
    /***********************************************************/

    printf("Today is: %s.\n", to_char(buffer,date(NULL),SPELLDATE));

    /***********************************************************/
    /* Do some simple date math.                               */
    /***********************************************************/

    dv1 = date(NULL) - to_date("4-Jul-1776", SPELLDATE);
    printf("%ld days since 4th of July 1776\n", dv1);

    /***********************************************************/
    /* Do some complex date math and work out strfdate().      */
    /* Compute 1 year, less 3 months, plus 2 weeks, less 3 days*/
    /* from the current date.                                  */
    /***********************************************************/

    dv1 = compute_date(date(NULL), 1, -3, 2, -3);
    dt = *localdate( &dv1 );
    strfdate( buffer, sizeof(buffer),
             "1 year, -3 months, 2 weeks, -3 days from today is: %A, %B %d %Y", &dt );
    puts( buffer );

    /***********************************************************/
    /* Test next_day_of_week()                                 */
    /***********************************************************/

    dv1 = date(NULL);
    puts("Testing next_day_of_week()\n");
    for (i = SUNDAY; i <= SATURDAY; i++) {
        dv2 = next_day_of_week( dv1, i );
        puts( to_char(buffer, dv2, SPELLDATE));
    }

    /***********************************************************/
    /* Test previous_day_of_week()                             */
    /***********************************************************/

    puts("\nTesting previous_day_of_week()\n");
    dv1 = previous_day_of_week( dv1, WEDNESDAY );
    for (i = SUNDAY; i <= SATURDAY; i++) {
        dv2 = previous_day_of_week( dv1, i );
        puts( to_char(buffer, dv2, SPELLDATE));
    }

    dv1 = to_date("04-Jul-1776", SPELLDATE);
    puts( to_char(buffer, next_day_of_week(dv1, SUNDAY), SPELLDATE));

    return EXIT_SUCCESS;
}
