/**********************************************************************/
/* File Id:                     today.c.                              */
/* Author:                      Stan Milam.                           */
/* Date Written:                31-Jan-1994.                          */
/*                                                                    */
/* This program receives one, possibly two, command line arguments    */
/* and computes a past or future date with the current date being     */
/* the basis for the date calculation.  The calculated date is then   */
/* displayed using a default format or a format determined by the     */
/* second command line argument. Following are some examples of the   */
/* program and its output.                                            */
/*                                                                    */
/* Command line                         Results                       */
/*                                                                    */
/* today 90                             Sunday, May 01 1994           */
/* today -90                            Tuesday, November 02 1994     */
/* today 45 "Today is: %d-%b-%y"        Today is: 17-Mar-94           */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <toolbox.h>

int main ( int argc, char **argv ) {

    struct  dt *dt;
    date_t  date_v;
    int     rv = EXIT_SUCCESS;
    char    buffer[BUFSIZ], *format = NULL;

    /******************************************************************/
    /* Check the command line to see if any arguments were used.  If  */
    /* none then tell user how this program works.                    */
    /******************************************************************/

    if ( argc < 2 ) {
        rv = EXIT_FAILURE;
        fputs("Usage: today [+/-]days [format string]\n", stderr);
    }
    else {
        /**************************************************************/
        /* Get the first argument which should be the number of days. */
        /**************************************************************/

        argv++;
        date_v = strtol(*argv, NULL, 0);

        /**************************************************************/
        /* Now see if a format statement was supplied.                */
        /**************************************************************/

        argv++;
        format = ( *argv == NULL ) ? "%A, %B %d %Y" : *argv;

        /**************************************************************/
        /* Now compute the new date. Check the return value to see if */
        /* we are out of range.                                       */
        /**************************************************************/

        date_v += date( NULL );
        if (( dt = localdate( &date_v )) == NULL ) {
            rv = EXIT_FAILURE;
            fputs("Value entered is out of range!\n", stderr);
        } 
        else {
            /**********************************************************/
            /* Format the message and print it.                       */
            /**********************************************************/
            strfdate(buffer, sizeof(buffer), format, dt);
            puts(buffer);
        }
    }
    return rv;
}
